/**
 *
 * A Lojban Text-to-Speech program.
 *
 * @version 1.0
 * @author michal wallace (http://www.sabren.net/)
 */

import java.util.*;
import javax.sound.sampled.*;

public class LojTalk implements LineListener {
    private boolean isPlaying;
    private static String [] alphabet = {
	"'", "A", "B", "C", "D", "E", 
	"F", "G", "I", "J", "K", "L", 
	"M", "N", "O", "P", "R", "S", 
	"T", "U", "V", "X", "Y", "Z"
    };
    private Hashtable phones;
    private static int inc = 0;

    /**
     * Loads a ready-to-play Clip from the specified filename.
     */
    public Clip loadClip(String filename) {
	Clip res;
	try {
	    AudioInputStream ais =
		AudioSystem.getAudioInputStream(new java.io.File(filename));
	    AudioFormat format = ais.getFormat();
	    
	    // can't open ALAW/ULAW playback line, so convert,
	    // (this is taken from jdk1.3/demo/sound/src/Juke.java
	    if ((format.getEncoding() == AudioFormat.Encoding.ULAW) ||
		(format.getEncoding() == AudioFormat.Encoding.ALAW)) {
		AudioFormat tmp = new AudioFormat(
					  AudioFormat.Encoding.PCM_SIGNED, 
					  format.getSampleRate(),
					  format.getSampleSizeInBits() * 2,
					  format.getChannels(),
					  format.getFrameSize() * 2,
					  format.getFrameRate(),
					  true);
		ais = AudioSystem.getAudioInputStream(tmp, ais);
		format = tmp;
	    }
	    
	    // now create the info to pass to getLine
	    DataLine.Info info = 
		new DataLine.Info(Clip.class, format,
				  ((int) ais.getFrameLength()
				   * format.getFrameSize()));

	    res = (Clip) AudioSystem.getLine(info);
	    res.open(ais);
	    res.addLineListener(this);
	    
	} catch (Exception e) {
	    //UnsupportedAudioFileException, IOExeption..
	    e.printStackTrace();
	    throw new Error("bad file?!");
	}
	return res;
    }

    public void update(LineEvent e) {
	if (e.getType() == LineEvent.Type.STOP) {
	    this.isPlaying = false;
	}
    }

    public void playClip(Clip c) {
	this.isPlaying = true;
	c.start();
	while (this.isPlaying) {} // wait around doing nothing...
	c.stop();
    }

    public void getPhones() {
	if (this.phones == null) {
	    this.phones = new Hashtable();
	    for (int i=0; i<alphabet.length; i++) {
		this.phones.put(alphabet[i], this.loadClip(alphabet[i]+".au"));
	    }
	}
    }

    public void closePhones() {
	if (this.phones != null) {
	    for (int i=0; i<alphabet.length; i++) {
		Clip c = (Clip) this.phones.get(alphabet[i]);
		c.close();
	    }
	}
    }

    public void pause(int time){
	try {
	    Thread.currentThread().sleep(time);
	} catch (InterruptedException e) {
	    e.printStackTrace();
	}
    }

    public static void main(String[] args) {
	LojTalk me = new LojTalk();
	me.getPhones();

	StringBuffer buff = new StringBuffer();
	for (int i=0; i<args.length; i++) {
	    buff.append(args[i]+" ");
	}
	
	String s = new String(buff).toUpperCase();
	for (int i=0; i<s.length(); i++) {
	    try {
		if (s.charAt(i) == ' ') {
		    me.pause(250);
		} else if (s.charAt(i) == '.') {
		    me.pause(500);
		} else {
		    me.playClip((Clip) me.phones.get(s.substring(i,i+1)));
		}
	    } catch (Exception e) {
		System.out.println(e);
	    }
	}
	me.closePhones();
	System.exit(0);
    }
}
